//
//  AASBasicInfoFileItem.h
//
//  Created by Andreas Schwarz on Thu Apr 18 2002.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#import <Cocoa/Cocoa.h>
#import "AASFileItem.h"

/* The AASBasicInfoFileItem is an AASFileItem subclass that contains some extra basic information about a file, such as the display name and whether it is a directory, application, package, and, if it is an alias or a symbolic link, the file it points to. */


#pragma mark Keys

/* Keys for archive dictionaries */
extern NSString *AASArchiveIsAliasKey;	// Will return an NSNumber BOOL value - YES if the file was a symbolic link or alias, NO if it was a 'real' file 




#pragma mark -


@interface AASBasicInfoFileItem : AASFileItem {
    char isApplication;
    char isDirectory;
    char isPackage;
    char isInvisible;
	char isAlias;
    
	char resolvedFSRefIsValid;
	FSRef resolvedFSRef;
	id resolvedFileItem;
	    
    NSString *displayName;
	NSString *kind;
	
    CFStringRef UTI;
	
	NSUInteger finderLabel;
	NSString *finderLabelTitle;
}


#pragma mark -
#pragma mark File Information

/* Returns the display name of the file (this takes into account whether or not the file extension is hidden, and returns the proper name for volumes (like the root volume). KVO compliant. */
@property(readonly) NSString * displayName;

/* Returns the UTI of the file. KVO compliant. */
@property(readonly) NSString *uti;

/* Returns the human-readable kind of the file. KVO compliant. */
@property(readonly) NSString *kindString;

/* Returns whether this file is an alias file. KVO compliant (but value should never change). */
@property(readonly) BOOL isAlias;

/* A shortcut to see whether the file is an alias/symlink or a "real" file. KVO compliant (but value should never change). */
@property(readonly) BOOL isRealFile;

/* Returns whether this file is an application.  This includes if it is an alias or symlink to an application. KVO compliant. */
@property(readonly) BOOL isApplication;

/* Returns whether this file is a directory.  This includes if it is an alias or symlink to a directory.  It excludes package. KVO compliant. */
@property(readonly) BOOL isDirectory;

/* Returns whether this file is a package.  This includes if it is an alias or symlink to a package. KVO compliant. */
@property(readonly) BOOL isPackage;

/* Returns whether this file is invisible. (Special cases: /net/ and /home/ and /dev/ are returned as invisible though LaunchServices does not always identify them as such. The Finder, however, does not show these items.) KVO compliant. */
@property(readonly) BOOL isInvisible;

/* Returns an integer value for the file's Finder label, or sets the file's label to a new value. KVO compliant. */
@property(assign) NSUInteger finderLabel;

/* Returns label text for the file's Finder label. KVO compliant. */
@property(readonly) NSString *finderLabelTitle;






#pragma mark -
#pragma mark Alias Information

/* If this file is an alias/symlink, this returns the path to whatever file the alias is pointing to.  Otherwise, it returns the path to this file. KVO compliant. */
@property(readonly) NSString * resolvedPath;

/* If this file is an alias/symlink, this returns the URL to whatever file the alias is pointing to.  Otherwise, it returns the URL to this file. KVO compliant. */
@property(readonly) NSURL * resolvedURL;

/* If the file is an alias/symlink, returns a pointer to an FSRef for the file the alias points to.  Otherwise returns a pointer to the FSRef for the file itself.  Might be null. KVO compliant. */
@property(readonly) FSRef * resolvedFSRef;

/* If the file is an alias/symlink, returns a AASBasicInfoFileItem instance (or subclass of the same class as the reciever) for the file the alias points to.  Otherwise returns itself. KVO compliant. */
@property(readonly) id resolvedFileItem;

/* If the file is an alias/symlink, returns the UTI for the file the alias points to.  Otherwise returns its own UTI. KVO compliant. */
@property(readonly) NSString *resolvedUTI;


#pragma mark -
#pragma mark Subclassable Methods

// Can be called to re-check the resolved FSRef in case this file is an alias and the file it pointed to is now gone.
// Returns YES if the file is an alias/symlink and NO otherwise
// Subclasses will want to implement this (make sure to call super) to reset any information that may hinge on what the alias points to
- (BOOL)resetAliasStatus;



#pragma mark -
#pragma mark Class Methods For File Filtering

/* Commonly used filter methods for convenience. Pass these to a file filter or directory filter along with setting the filter's target to [AASBasicInfoFileItem class]. */

/* Filters out invisible files. */
+ (NSNumber *)filterByVisibility:(AASBasicInfoFileItem *)file;

/* Includes regular folders and aliases to regular folders, but not packages or bundles. Also filters out invisible files. */
+ (NSNumber *)filterByNormalDirectory:(AASBasicInfoFileItem *)file;

/* Returns the resolved file item for aliases, or just returns the file item for non-aliases. */
+ (AASBasicInfoFileItem *)trueDirectoryForNormalDirectory:(AASBasicInfoFileItem *)file;

@end
